#include "gfx/entity_object.hpp"

#include "gfx/mesh.hpp"
#include "gfx/surface.hpp"

using namespace gfx;

EntityObject::EntityObject()
{
	this->enableDraw();
}

EntityObject::EntityObject(const math::vec3d &ppos, const math::vec3d &prot) :
	Entity(ppos, prot)
{
	this->enableDraw();
}

EntityObject::EntityObject(const math::vec3d &ppos, const math::vec3d &prot,
		const Mesh *pmesh) :
	Entity(ppos, prot)
{
	this->enableDraw();
	this->addMesh(pmesh);
}

EntityObject::EntityObject(const Mesh *pmesh)
{
	enableDraw();
	this->addMesh(pmesh);
}

void EntityObject::draw(const Shader &psh, const math::mat4f &pmat)
{
	if(draw_disabled) return;

	math::mat4f tm = this->orient(pmat);

	BOOST_FOREACH(const Mesh* vv, m_meshes)
	{
		vv->draw(psh, tm);
	}
	BOOST_FOREACH(const EntityObjectSptr &vv, m_objects)
	{
		vv->draw(psh, tm);
	}
}

bool EntityObject::isDrawDisabled()
{
	return draw_disabled;
}

math::mat4f EntityObject::orient(const math::mat4f &pmat)
{
	math::mat4f ret = pmat * m_wm;
	load_modelview(ret);
	return ret;
}

