#include "gfx/mesh_static.hpp"

#include "gfx/shader.hpp"
#include "thr/dispatch.hpp"

using namespace gfx;

MeshStatic::MeshStatic(const std::string &pfname)
{
	this->load(pfname);
}

void MeshStatic::compile()
{
	// Texcoords is sometimes smaller than the others.
	if(_texcoord.size() < _vertex.size())
	{
		_texcoord.resize(_vertex.size(), math::vec2f(0.0f, 0.0f));
	}

	this->calcNormals();
	thr::dispatch_privileged(boost::bind(&BufferInterleavedTCNV::update, &_buf,
				boost::ref(_texcoord), boost::ref(_color), boost::ref(_normal),
				boost::ref(_vertex)));
	thr::wait_privileged(boost::bind(&Mesh::taskElem, this));
	_lod.compile(_vertex);
}

void MeshStatic::draw(const Shader &psh, const math::mat4f &pmat) const
{
	if(!_textures.empty())
	{
		_textures[0]->bind(psh.getUniformTex(), 0);
	}
	_buf.feed(psh.getAttrTexcoord(), psh.getAttrColor(), psh.getAttrNormal(),
  		psh.getAttrVertex());
	_elem.bind();
	_lod.draw(pmat);
}

void MeshStatic::unreserve()
{
	_buf.unreserve();
	this->Mesh::unreserve();
}

