#include "ob_city.hpp"

#include "math/random.hpp"
#include "ob_constants.hpp"
#include "ob_game.hpp"
#include "ob_globals.hpp"
#include "ob_height_map_planet.hpp"

using namespace ob;

/** Population scale factor (for transforming stuff in volumes). */
static const float OB_POPULATION_CITY_BRUSH = 3.0f;

/** Random population instance count. */
static const int OB_POPULATION_CITY_OUTSKIRTS_MAX = 48;

/** Random population instance count. */
static const int OB_POPULATION_CITY_OUTSKIRTS_MIN = 24;

/** Population scale factor (for transforming stuff in volumes). */
static const float OB_POPULATION_OUTSKIRTS_BRUSH = 1.8f;

City::City(PopulationMap &vol, const HeightMapPlanet &hmap) :
	gfx::EntityObjectOverlay(),
	Billboard(&(glob->getTextureIconCity()))
{
	m_pos = randomize_pos_on_surface(hmap);
	m_wm = lookat_on_surface(m_pos, OB_CITY_MARKER_HEIGHT);

	vol.paint(math::vec3f(m_pos), OB_POPULATION_CITY_BRUSH);

	for(int ii = 0, cap = math::mrand(OB_POPULATION_CITY_OUTSKIRTS_MIN, OB_POPULATION_CITY_OUTSKIRTS_MAX); (ii <= cap); ++ii)
	{
		math::vec3f rndvec(math::mrand(-OB_POPULATION_OUTSKIRTS_PROXIMITY, OB_POPULATION_OUTSKIRTS_PROXIMITY),
				math::mrand(-OB_POPULATION_OUTSKIRTS_PROXIMITY, OB_POPULATION_OUTSKIRTS_PROXIMITY),
				math::mrand(-OB_POPULATION_OUTSKIRTS_PROXIMITY, OB_POPULATION_OUTSKIRTS_PROXIMITY)),
			rndpos = math::vec3f(m_pos) + rndvec;
		if(hmap.calcHeight(rndpos) > OB_TERRAIN_LEVEL)
		{
			vol.paint(rndpos, OB_POPULATION_OUTSKIRTS_BRUSH);
		}
	}
}

City::~City()
{
	if(m_silos.size() > 0)
	{
		std::stringstream sstr;
		sstr << m_silos.size() << " silos remaining at city " << this
			<< " destruction";
		BOOST_THROW_EXCEPTION(std::runtime_error(sstr.str()));
	}
}

const gfx::Texture2D* City::fillBillboard(const gfx::Texture2D *tex) const
{
	if(!this->isVisible())
	{
		return NULL;
	}

	const gfx::Texture2D *ret = this->updateBillboard(tex);
	gfx::billboard_fill(0xFFFFFFFF, math::vec3f(m_pos), OB_BILLBOARD_SIZE_CITY);
	return ret;
}

void City::removeSilo(const Silo *silo)
{
	size_t sptr = reinterpret_cast<size_t>(silo);
	std::map<size_t, Silo*>::iterator iter = m_silos.find(sptr);
	if(m_silos.end() == iter)
	{
		std::stringstream sstr;
		sstr << "requested to remove non-existing silo " << silo <<
			" from city " << this;
		BOOST_THROW_EXCEPTION(std::runtime_error(sstr.str()));
	}
	m_silos.erase(iter);
}

void City::update()
{
	float len2rpos = math::length2(m_pos - game->getView().getPos());
	this->updateVisibility(len2rpos <= OB_CITY_CULL * OB_CITY_CULL);

	if((m_silos.size() < OB_SILOS_PER_CITY) &&
			(math::mrand(1, OB_SILO_POPUP_PROBABILITY) == 1))
	{
		Silo *silo(new Silo(glob->getHeightMapPlanet(), this));
		size_t sptr = reinterpret_cast<size_t>(silo);
		m_silos[sptr] = silo;
		game->addSilo(silo);
		//std::cout << "creating silo " << m_silos.size() << " for city " << this << std::endl;
	}
}

