#ifndef OB_CITY_HPP
#define OB_CITY_HPP

#include "gfx/entity_object_overlay.hpp"
#include "ob_billboard.hpp"
#include "ob_surface_element.hpp"
#include "ob_visualization.hpp"

#include <map>

namespace ob
{
	class PopulationMap;
	class HeightMapPlanet;
	class Silo;

	/** \brief Representation of a city.
	 */
	class City :
		public gfx::EntityObjectOverlay,
		public Billboard,
		public Visualization,
		public SurfaceElement
	{
		private:
			/** \brief Silos for this city.
			 *
			 * Indexed by the contents of the smart pointer.
			 */
			std::map<size_t, Silo*> m_silos;

		public:
			/** \brief Accessor.
			 *
			 * @return Silo map.
			 */
			std::map<size_t, Silo*>& getSilos()
			{
				return m_silos;
			}

			/** \brief Accessor.
			 *
			 * @return Silo map.
			 */
			const std::map<size_t, Silo*>& getSilos() const
			{
				return m_silos;
			}

		public:
			/** \brief Create a new city.
			 *
			 * Position is randomized from the heightmap.
			 *
			 * @param vol Volume to store population data into.
			 * @param hmap Height map.
			 */
			City(PopulationMap &vol, const HeightMapPlanet &hmap);

			/** \brief Destructor. */
			virtual ~City();

		public:
			/** \brief Remove a silo from this.
			 *
			 * @param silo Silo to remove.
			 */
			void removeSilo(const Silo *silo);

			/** \brief Gamistic update.
			 */
			void update();

		public:
			/** \cond */
			virtual const gfx::Texture2D* fillBillboard(const gfx::Texture2D *tex) const;
			/** \endcond */

		private:
			/** \brief Generate a random position from heightmap.
			 *
			 * @param hmap.
			 * @return Reference to the new position 
			 */
			const math::vec3f& randomizePos(const HeightMapPlanet &hmap);
	};
}

#endif
