#include "ob_height_map_planet.hpp"

#include "ob_constants.hpp"

using namespace ob;

HeightMapPlanet::HeightMapPlanet() :
	gfx::HeightMapBall("gfx/maps/surf_bk.png", "gfx/maps/surf_dn.png",
			"gfx/maps/surf_fw.png", "gfx/maps/surf_lt.png",
			"gfx/maps/surf_rt.png", "gfx/maps/surf_up.png",
			OB_PLANET_RADIUS * (1.0f - OB_PLANET_RADIUS_DIFF),
			OB_PLANET_RADIUS * (1.0f + OB_PLANET_RADIUS_DIFF)) { }

gfx::Color HeightMapPlanet::calcColor(const math::vec3f &pnt, float ht) const
{
#if 0
	const float
		baseline[] = { 0.3f, 0.8f, 0.3f },
		mountain[] = { 0.7f, 0.5f, 0.1f },
		arctic[] = { 0.9f, 0.9f, 0.9f },
		desert[] = { 0.8f, 0.4f, 0.1f },
		water[] = { 0.1f, 0.1f, 0.8f };
	float ret[] = { 0.0f, 0.0f, 0.0f };

	ret[0] = baseline[0];
	ret[1] = baseline[1];
	ret[2] = baseline[2];

	if(ht > 1.0f)
	{
		std::stringstream sstr;
		sstr << ht;
		BOOST_THROW_EXCEPTION(std::runtime_error(sstr.str()));
	}

	if(ht > 0.5f)
	{
		ht = (ht - 0.5f) / 0.5f;
		ret[0] = ret[0] * (1.0f - ht) + mountain[0] * ht;
		ret[1] = ret[1] * (1.0f - ht) + mountain[1] * ht;
		ret[2] = ret[2] * (1.0f - ht) + mountain[2] * ht;

		float lat = (math::abs(pnt.y()) - 0.5f);
		if(lat > 0.0f)
		{
			ret[0] = ret[0] * (1.0f - lat) + arctic[0] * lat;
			ret[1] = ret[1] * (1.0f - lat) + arctic[1] * lat;
			ret[2] = ret[2] * (1.0f - lat) + arctic[2] * lat;
		}
		else
		{
			ret[0] = ret[0] * (1.0f + lat) + desert[0] * (-lat);
			ret[1] = ret[1] * (1.0f + lat) + desert[1] * (-lat);
			ret[2] = ret[2] * (1.0f + lat) + desert[2] * (-lat);
		}
	}
	else
	{
		ht = math::sqrt((0.5f - ht) / 0.5f);
		ret[0] = ret[0] * (1.0f - ht) + water[0] * ht;
		ret[1] = ret[1] * (1.0f - ht) + water[1] * ht;
		ret[2] = ret[2] * (1.0f - ht) + water[2] * ht;
	}
	return Color(ret[0], ret[1], ret[2], 1.0f);
#else
	float lat = math::abs(pnt.y());
	lat *= lat;
	if(ht < 0.5f)
	{
		ht = (ht - 0.45f) / 0.05f;
	}
	else
	{
		ht = 1.0f + math::sqrt((ht - 0.5f) / 0.5f);
	}
	ht *= 0.5f;
	return gfx::Color(ht, ht, ht, lat);
#endif
}

float HeightMapPlanet::calcHeight(const math::vec3f &vv) const
{
	return std::max(this->gfx::HeightMapBall::calcHeight(vv), 0.45f);
}

